local L = LibStub("AceLocale-3.0"):GetLocale("ShockAndAwe")
local media = LibStub:GetLibrary("LibSharedMedia-3.0")

ShockAndAwe.ExportFrame = CreateFrame("Frame", "SAA_ExportFrame", UIParent)

function ShockAndAwe:CreateExportFrame()
	self.ExportFrame:SetFrameStrata("BACKGROUND")
	self.ExportFrame:SetWidth(400)
	self.ExportFrame:SetHeight(500)
	self.ExportFrame:SetBackdrop(self.frameBackdrop)
	self.ExportFrame:SetBackdropColor(0, 0, 0, 1);
	self.ExportFrame:SetMovable(true);
	self.ExportFrame:RegisterForDrag("linesButton");
	self.ExportFrame:SetScript("OnDragStart", 
		function()
			self.ExportFrame:StartMoving();
		end );
	self.ExportFrame:SetScript("OnDragStop",
		function()
			self.ExportFrame:StopMovingOrSizing();
			self.ExportFrame:SetScript("OnUpdate", nil);
		end );
	self.ExportFrame:SetPoint("CENTER")

	if not self.ExportFrame.titleText then
		self.ExportFrame.titleText = self.ExportFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlightLarge")
	end
	self.ExportFrame.titleText:SetPoint("TOP", 0, -5)
	
	if not self.ExportFrame.text then
		self.ExportFrame.text = self.ExportFrame:CreateFontString(nil, "OVERLAY")
	end
	self.ExportFrame.text:SetTextColor(1,1,1,1)
	self.ExportFrame.text:SetFont("Friz Quadrata TT", 12)
	self.ExportFrame.text:SetPoint("TOP", self.ExportFrame, "TOP")
	self.ExportFrame.lines = {}
	self.ExportFrame:Clear()
end

function ShockAndAwe.ExportFrame:AddLine(text)
	self.ExportFrame.lines[#self.ExportFrame.lines+1] = trim(text);
end

function ShockAndAwe.ExportFrame:Clear()
	for i = 1, #self.ExportFrame.lines do
		self.ExportFrame.lines[i] = nil
	end
end

function ShockAndAwe.ExportFrame:Display()
	local maxWidth = 0
	local textHeight = 0
	for i = 1, #self.ExportFrame.lines do
		if not self.ExportFrame.lines[i] then
			local lines = self.ExportFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
			lines:SetFont("Friz Quadrata TT", 12, "OUTLINE");
			self.ExportFrame.lines[i] = lines
			if i == 1 then
				lines:SetPoint("TOPRIGHT", self.ExportFrame, "TOPlines", 75, -35)
			else
				lines:SetPoint("TOPRIGHT", self.ExportFrame.lines[i-1], "BOTTOMRIGHT", 0, -1)
			end
		end
		self.ExportFrame.lines[i]:SetText(self.liness[i] .. ((len(self.liness[i]) > 0 and len(strtrim(self.liness[i])) > 0 and ":") or " "))
		local linesWidth = self.ExportFrame.lines[i]:GetWidth()
		textHeight = self.ExportFrame.lines[i]:GetHeight()
		if maxlinesWidth < linesWidth then
			maxlinesWidth = linesWidth
		end
	end
	for i = #self.liness+1, #self.ExportFrame.lines do
		self.ExportFrame.lines[i]:SetText('')
	end
	local finalWidth = maxlinesWidth + 10
	if (finalWidth < 360) then 
		finalWidth = 360
	end
	self.ExportFrame:SetWidth(finalWidth)
	self.ExportFrame:SetHeight(#self.liness * (textHeight + 1) + 100)
	self.ExportFrame:Show()
end

function ShockAndAwe:ExportToSim()
	local rating, percentage
	self:CreateExportFrame()
	self.ExportFrame:Clear()
	
	self.ExportFrame:AddLine("########################")
	self.ExportFrame:AddLine("## ShockAndAwe EnhSim Export ##")
	self.ExportFrame:AddLine("########################")
	
	local mh, oh = UnitAttackSpeed("player")
	self.ExportFrame:AddLine("mh_speed             "..mh)
	self.ExportFrame:AddLine("oh_speed             "..mh)
	
	-- mh_dps
	-- oh_dps
	
	percentage = GetCombatRatingBonus(9) -- Melee Crit rating
	self.ExportFrame:AddLine("mh_crit             "..percentage)
	self.ExportFrame:AddLine("oh_crit             "..percentage)
	self.ExportFrame:AddLine("##getcrit             "..GetCritChance())
	
	percentage = GetCombatRatingBonus(6) -- Melee Hit rating
	self.ExportFrame:AddLine("mh_hit              "..percentage)
	self.ExportFrame:AddLine("oh_hit              "..percentage)
	
	rating = GetCombatRating(24) -- Expertise rating
	self.ExportFrame:AddLine("mh_expertise_rating "..rating)
	self.ExportFrame:AddLine("oh_expertise_rating "..rating)
	
	self.ExportFrame:AddLine("ap                   "..UnitAttackPower("player"))
	
	percentage = GetCombatRatingBonus(18) -- Haste rating
	self.ExportFrame:AddLine("haste               "..percentage)
	
	percentage = GetCombatRatingBonus(25) -- Armour Penetration rating
	self.ExportFrame:AddLine("armor_penetration   "..percentage)
	
	self.ExportFrame:AddLine("str                  "..UnitStat("player",1))
	self.ExportFrame:AddLine("agi                  "..UnitStat("player",2))
	self.ExportFrame:AddLine("int                  "..UnitStat("player",4))
	
	self.ExportFrame:AddLine("spellpower        "..GetSpellBonusDamage(4))
	
	percentage = GetCombatRatingBonus(11) -- Spell crit rating
	self.ExportFrame:AddLine("spell_crit          "..percentage)
	self.ExportFrame:AddLine("##getspell_crit          "..GetSpellCritChance(4))

	percentage = GetCombatRatingBonus(8) -- Spell hit rating
	self.ExportFrame:AddLine("spell_hit           "..percentage)
	
	-- base mana
	
	if self.db.char.binding.mhspell == L["Windfury Weapon"] then
		self.ExportFrame:AddLine("mh_imbue        windfury")
	elseif self.db.char.binding.mhspell == L["Flametongue Weapon"] then
		self.ExportFrame:AddLine("mh_imbue        flametongue")
	else
		self.ExportFrame:AddLine("mh_imbue        -")
	end
	
	if self.db.char.binding.ohspell == L["Windfury Weapon"] then
		self.ExportFrame:AddLine("oh_imbue        windfury")
	elseif self.db.char.binding.ohspell == L["Flametongue Weapon"] then
		self.ExportFrame:AddLine("oh_imbue        flametongue")
	else
		self.ExportFrame:AddLine("oh_imbue        -")
	end
	
	self.ExportFrame:Display()
end

function ShockAndAwe:GetTrinketParameter(slot)

end