local L = LibStub("AceLocale-3.0"):GetLocale("ShockAndAwe")
local media = LibStub:GetLibrary("LibSharedMedia-3.0");

ShockAndAwe.PriorityFrame = CreateFrame("Frame", "SAA_PriorityFrame", UIParent)

---------------------------
-- Local Variables
---------------------------

-- constants
ShockAndAwe.priorityType = {}
ShockAndAwe.priorityType.MW5_LB = "mw5_lb"
ShockAndAwe.priorityType.MW5_CL = "mw5_cl"
ShockAndAwe.priorityType.SS = "ss"
ShockAndAwe.priorityType.ES = "es"
ShockAndAwe.priorityType.FS = "fs"
ShockAndAwe.priorityType.LL = "ll"

ShockAndAwe.priorityTable = {}
ShockAndAwe.priorityTable.name = {}
ShockAndAwe.priorityTable.icon = {}
ShockAndAwe.priorityTable.test = {}
ShockAndAwe.priorityTable.name["mw5_lb"] = L["MW5 Lightning Bolt"]
_, _, ShockAndAwe.priorityTable.icon["mw5_lb"] = GetSpellInfo(49238)
ShockAndAwe.priorityTable.test["mw5_lb"] = 
	function () 
		return ShockAndAwe.db.char.msstacks == 5 
	end 

ShockAndAwe.priorityTable.name["mw5_cl"] = L["MW5 Chain Lightning"]
_, _, ShockAndAwe.priorityTable.icon["mw5_cl"] = GetSpellInfo(49269)
ShockAndAwe.priorityTable.test["mw5_cl"] = 
	function () 
		local start, duration = GetSpellCooldown(L["Chain Lightning"])
		local timeleft = start + duration - GetTime()
		return ShockAndAwe.db.char.msstacks == 5 and timeleft <= ShockAndAwe.db.char.priority.cooldown 
	end 

ShockAndAwe.priorityTable.name["ss"] = L["Stormstrike"]
_, _, ShockAndAwe.priorityTable.icon["ss"] = GetSpellInfo(17364)
ShockAndAwe.priorityTable.test["ss"] =  
	function () 
		local start, duration = GetSpellCooldown(L["Stormstrike"])
		local timeleft = start + duration - GetTime()
		return timeleft <= ShockAndAwe.db.char.priority.cooldown 
	end 

ShockAndAwe.priorityTable.name["es"] = L["Earth Shock"]
_, _, ShockAndAwe.priorityTable.icon["es"] = GetSpellInfo(49231)
ShockAndAwe.priorityTable.test["es"] =  
	function () 
		local start, duration = GetSpellCooldown(L["Earth Shock"])
		local timeleft = start + duration - GetTime()
		return timeleft <= ShockAndAwe.db.char.priority.cooldown 
	end 

ShockAndAwe.priorityTable.name["fs"] = L["Flame Shock"]
_, _, ShockAndAwe.priorityTable.icon["fs"] = GetSpellInfo(49233)
ShockAndAwe.priorityTable.test["fs"] =  
	function () 
		local start, duration = GetSpellCooldown(L["Flame Shock"])
		local timeleft = start + duration - GetTime()
		return timeleft <= ShockAndAwe.db.char.priority.cooldown 
	end 
ShockAndAwe.priorityTable.name["ll"] = L["Lava Lash"]
_, _, ShockAndAwe.priorityTable.icon["ll"] = GetSpellInfo(60103)
ShockAndAwe.priorityTable.test["ll"] =  
	function () 
		local start, duration = GetSpellCooldown(L["Lava Lash"])
		local timeleft = start + duration - GetTime()
		return timeleft <= ShockAndAwe.db.char.priority.cooldown 
	end 

ShockAndAwe.priorityTable.name["none"] = L["none"]
_, _, ShockAndAwe.priorityTable.icon["none"] = "Interface/Tooltips/UI-Tooltip-Background"
ShockAndAwe.priorityTable.test["none"] = function () return false end 

-------------------------------
-- Priority Frame functions
-------------------------------

function ShockAndAwe:SetPriorityBackdrop(icon)
	self.PriorityFrame:SetBackdrop({ bgFile = icon,
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = false, tileSize = 0, edgeSize = 12,
		insets = { left = 2, right = 2, top = 2, bottom = 2 }
	})
end

function ShockAndAwe:CreatePriorityFrame()
	self.PriorityFrame:SetScale(self.db.char.priority.scale)
	self.PriorityFrame:SetFrameStrata("BACKGROUND")
	self.PriorityFrame:SetWidth(self.db.char.priority.fWidth)
	self.PriorityFrame:SetHeight(self.db.char.priority.fHeight)
	self:SetPriorityBackdrop("Interface/Tooltips/UI-Tooltip-Background")
	self.PriorityFrame:SetBackdropColor(0, 0, 0, self.db.char.priority.alpha);
	self.PriorityFrame:SetMovable(true);
	self.PriorityFrame:RegisterForDrag("LeftButton");
	self.PriorityFrame:SetPoint(self.db.char.priority.point, self.db.char.priority.relativeTo, self.db.char.priority.relativePoint, self.db.char.priority.xOffset, self.db.char.priority.yOffset)
	self.PriorityFrame:SetScript("OnDragStart", 
		function()
			self.PriorityFrame:StartMoving();
		end );
	self.PriorityFrame:SetScript("OnDragStop",
		function()
			self.PriorityFrame:StopMovingOrSizing();
			self.PriorityFrame:SetScript("OnUpdate", nil);
			self:FinishedMoving(self.db.char.priority, self.PriorityFrame);
		end );
		
	if not self.PriorityFrame.topText then
		self.PriorityFrame.topText = self.PriorityFrame:CreateFontString(nil, "OVERLAY")
	end
	self.PriorityFrame.topText:SetTextColor(1,1,1,1)
	self.PriorityFrame.topText:SetFont(media:Fetch("font", self.db.char.barfont), self.db.char.barfontsize)
	self.PriorityFrame.topText:SetPoint("TOP", self.PriorityFrame, "TOP", 0, self.db.char.barfontsize + 2)
	self.PriorityFrame.topText:SetText(L["Next Priority"])

	if self.db.char.priority.show and InCombatLockdown() then
		self.PriorityFrame:Show()
	else
		self.PriorityFrame:Hide()
	end
	if self.db.char.priority.titleshow then
		self.PriorityFrame.toptext:Show()
	else
		self.PriorityFrame.toptext:Hide()
	end
end

function ShockAndAwe:ResetPriority()
	self.PriorityFrame:ClearAllPoints()
	self.db.char.Priority.point = defaults.char.priority.point
	self.db.char.priority.relativeTo = defaults.char.priority.relativeTo 
	self.db.char.priority.relativePoint = defaults.char.priority.relativePoint
	self.db.char.priority.xOffset = defaults.char.priority.xOffset
	self.db.char.priority.yOffset = defaults.char.priority.yOffset
	self.db.char.priority.fWidth = defaults.char.priority.fWidth
	self.db.char.priority.fHeight = defaults.char.priority.fHeight
	self.db.char.priority.scale = defaults.char.priority.scale
	self.PriorityFrame:SetPoint(self.db.char.priority.point, self.db.char.priority.relativeTo, self.db.char.priority.relativePoint, self.db.char.priority.xOffset, self.db.char.priority.yOffset)
	self:CreatePriorityFrame()
	self.db.char.barshow = true -- so that call to ShowHideBars resets to false
	self:ShowHideBars()
	self:Print(L["priority_reset"])
end

function ShockAndAwe:SetPriorityIcon(priority)
	if InCombatLockdown() or priority == "none" then
		local icon = self.priorityTable.icon[priority]
		if self.db.char.debug then
			self:Print("next : "..self.db.char.priority.next.." priority:"..priority)  
		end
		if self.db.char.windshockshow and (GetNumPartyMembers()>=4 or GetNumRaidMembers()>=4) then
			-- don't use windshock option if soloing
			local _, state, _, raw = UnitDetailedThreatSituation("player", "target")
			-- state 2 or 3 = tanking which is not what an Enh Shammy should be doing
			-- state 0 threat < 100%, state 1 threat >=100%
			if not state then state = 0 end
			if not raw then raw = 0 end
			if  state > 1 or raw > self.db.char.threatThreshold then
				-- we have exceeded threshold or have already started tanking so wind shock needed.
				_, _, icon = GetSpellInfo(57994)
			end
		end
		self:SetPriorityBackdrop(icon)
	end
end

function ShockAndAwe:SetNextPriority()
	if self.priorityTable.test[self.db.char.priority.option[1]]() then
		self.db.char.priority.next = self.db.char.priority.option[1]
	elseif self.priorityTable.test[self.db.char.priority.option[2]]() then
		self.db.char.priority.next = self.db.char.priority.option[2]
	elseif self.priorityTable.test[self.db.char.priority.option[3]]() then
		self.db.char.priority.next = self.db.char.priority.option[3]
	elseif self.priorityTable.test[self.db.char.priority.option[4]]() then
		self.db.char.priority.next = self.db.char.priority.option[4]
	elseif self.priorityTable.test[self.db.char.priority.option[5]]() then
		self.db.char.priority.next = self.db.char.priority.option[5]
	else
		self.db.char.priority.next = "none"
	end	
	if self.db.char.debug then
		self:Print("next set to :"..self.db.char.priority.next)
	end
end